import PyQt4.QtCore
import PyQt4.QtGui

import Constants
import Widget
import Holders

class AbstractStatistics (Widget.DockWidget):
	def __init__ (self, parent, bridge):
		self._type = "Statistics"
		Widget.DockWidget.__init__ (self, parent, bridge)

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		transformButton = Constants.BQPushButton ("Execute", layout)
		PyQt4.QtCore.QObject.connect (transformButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.execute)

		clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		self.list =  Holders.FreeHolderGroup (self._bridge, self._inList, self._outList)
		layout = self.list.layout
		outline.setLayout (layout)
		return outline

	def clear (self):
		self.list.clear ()
		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			attr = self._bridge._regionViewer.getSingleSelectedItem ()

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or \
		(attr.Type () == "VALUE" and attr._type == "_basin.Attribute"):
			for holder in self.list._inList:
				if holder.SetItem (attr, clobber = False) == True:
					return
		elif attr.Type () == "LIST":
			if self.list.text () == "None":
				self.list.setText (attr.text (0))
				self.list._shellAccess = attr._shellAccess

	def setOutput (self, state):
		self.clear ()
		if   state == 0:
			self.list.setEnabled (False)
		elif state == 2:
			self.list.setEnabled (True)

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inAttributes = ""
		result = self.list.getInAccesses ()
		if result == False:
			return
		for attr in result:
			inAttributes = str (attr)
		outAttributes = ""
		result = self.list.getOutNames ()
		if result == False:
			return
		for attr in result:
			outAttributes = str (attr)

		commands = []
		for out in self.list.getOutNames ():
			commands.append (str (out) + " = Attribute ()")

		commands.append (outAttributes + " = " + \
			self._funcName + " (" + inAttributes + ")")

		self._bridge._ipyShell.BatchCommands (commands, True)

		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()
		self.list.clear ()


class Mean (AbstractStatistics):
	def __init__ (self, parent, bridge):
		self._funcName = "mean"
		self._displayName = "Mean"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/statistics/mean.png")
		self._description = "Simply place an attribute in the `in` holder.  The `mask` holder is optional. " + \
			"Clicking the `Execute` will create a new free variable!"
		self._inList = ["in", "mask"]
		self._outList = ["out"]
		AbstractStatistics.__init__ (self, parent, bridge)

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		_in = self.list.getInAccesses (False)
		_out = self.list.getOutNames (False)
		if len (_in) == 0 or len (_out) == 0:
			return False

		begin = _out[0] + " = " + self._funcName + "(" + _in[0]

		commands = []
		if   len (_in) == 1:
			commands.append (begin + ")")
		elif len (_in) == 2:
			commands.append (begin + ", " + _in[1] + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()
		self.list.clear ()


class StandardDeviation (AbstractStatistics):
	def __init__ (self, parent, bridge):
		self._funcName = "stdev"
		self._displayName = "Standard Deviation"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/statistics/stddev.png")
		self._description = "Simply place an attribute in the `in` holder.  The `mask` holder is optional. Clicking the `Execute` will create a new free variable!"
		self._inList = ["in", "mask"]
		self._outList = ["out"]
		AbstractStatistics.__init__ (self, parent, bridge)

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		_in = self.list.getInAccesses (False)
		_out = self.list.getOutNames (False)
		if len (_in) == 0 or len (_out) == 0:
			return False

		begin = _out[0] + " = " + self._funcName + "(" + _in[0]

		commands = []
		if   len (_in) == 1:
			commands.append (begin + ")")
		elif len (_in) == 2:
			commands.append (begin + ", " + _in[1] + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()
		self.list.clear ()


class Histogram (AbstractStatistics):
	def __init__ (self, parent, bridge):
		self._funcName = "histogram"
		self._displayName = "Histogram"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/statistics/histogram.png")
		self._description = "Simply place an attribute in the `in` holder.  The `mask` holder is optional. " + \
			"Clicking the `Execute` will create a new free variable!"
		self._inList = ["in"]
		self._outList = ["out"]
		AbstractStatistics.__init__ (self, parent, bridge)

		self._binmin = PyQt4.QtGui.QDoubleSpinBox ()
		self._binmin.setRange (-999999999.9, 999999999.9)
		self._dbin = PyQt4.QtGui.QDoubleSpinBox ()
		self._dbin.setRange (0.0, 999999999.9)
		self._nbins = PyQt4.QtGui.QSpinBox ()
		self._nbins.setRange (1, 999999999)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("bin min:"), 2, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self._binmin, 2, 1, PyQt4.QtCore.Qt.AlignLeft)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("d bin:"), 3, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self._dbin, 3, 1, PyQt4.QtCore.Qt.AlignLeft)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("n bins:"), 4, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self._nbins, 4, 1, PyQt4.QtCore.Qt.AlignLeft)

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		_in = self.list.getInAccesses (False)
		_out = self.list.getOutNames (False)
		if len (_in) == 0 or len (_out) == 0:
			return False

		begin = _out[0] + " = " + self._funcName + "(" + _in[0]

		commands = []
		commands.append (begin + ", " + str (self._binmin.value ()) + \
			", " + str (self._dbin.value ()) + ", " + str (self._nbins.value ()) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()
		self.list.clear ()
