import PyQt4.QtCore
import PyQt4.QtGui

import Constants
import DataGuesser
import Widget
import Holders

class AbstractStellarDynamics (Widget.DockWidget):
	def __init__ (self, parent, bridge):
		self._type = "Stellar Dynamics"
		Widget.DockWidget.__init__ (self, parent, bridge)

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout ()
		box.setLayout (layout)

		executeButton = Constants.BQPushButton ("Perform", layout)
		PyQt4.QtCore.QObject.connect (executeButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.perform)

		clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		self.list =  Holders.FreeHolderGroup (self._bridge, self._inList, self._outList)
		outline.setLayout (self.list.layout)

		return outline

	def clear (self):
		self.list.clear ()
		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" \
		or (attr.Type () == "VALUE" and attr._type == "_basin.Attribute"):
			for holder in self.list._inList:
				if holder.SetItem (attr, clobber = False) == True:
					return True
		return False

	def setOutput (self, state):
		self.clear ()
		if   state == 0:
			self.list.setEnabled (False)
		elif state == 2:
			self.list.setEnabled (True)

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inAttributes = ""
		result = self.list.getInAccesses ()
		if result == False:
			return
		for attr in result:
			inAttributes += ", " + str (attr)
		outAttribute = ""
		result = self.list.getOutNames ()
		if result == False:
			return
		for attr in result:
			outAttribute = str (attr)

		commands = []
		for out in self.list.getOutNames ():
			commands.append (str (out) + " = Attribute ()")

		commands.append (str (outAttribute) + " = " + \
			self._funcName + " (" + str (inAttributes[2:]) + ")")

		self._bridge._ipyShell.BatchCommands (commands, True)

		for holder in self.list._inList:
			holder.clear ()
		for holder in self.list._outList:
			holder.clear ()
		self.list.clear ()


class CenterOfMass (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Center of Mass"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/stellar_dynamics/centerofmass.png")
		self._description = ""
		self._inList = ["x", "y", "z"]
		self._outList = ["cm_out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)
		self.m = Holders.ChildAttributeHolder (bridge, self.list)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("mass:"), 4, 0)
		self.list.layout.addWidget (self.m, 4, 1)

	def guessDefaults (self):
		guessList = []
		for item in self._inList:
			guessList.append (DataGuesser.GenerateRegExpFromName (item))
		guessList.append (DataGuesser.GenerateRegExpFromWord ("MASS"))
		DataGuesser.GuessAttributes (self, guessList)

	def clear (self):
		AbstractStellarDynamics.clear (self)
		self.m.clear ()

	def DoubleClicked (self, attr = None):
		if not AbstractStellarDynamics.DoubleClicked (self, attr):
			if self.m.SetItem (attr, clobber = False) == True:
				return True
		return False

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inAttributes = ""
		inResult = self.list.getInAccesses (False)
		if inResult == False:
			return
		for attr in inResult:
			inAttributes += ", " + str (attr)
		outAttributes = ""
		outResult = self.list.getOutNames (False)
		if outResult == False:
			return
		for attr in outResult:
			outAttributes = str (attr)

		commands = []
		if   len (inResult) == 2 and len (outResult) == 1:
			commands.append (str (outAttributes) + \
				" = center_of_mass_2d (" + str (inAttributes[2:]) + ", " + \
				str (self.m._item._shellAccess) + ")")
		elif len (inResult) == 3 and len (outResult) == 1:
			commands.append (str (outAttributes) + \
				" = center_of_mass (" +	str (inAttributes[2:]) + ", " + \
				str (self.m._item._shellAccess) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)


class LocalDensity (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Local Density"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = ""
		self._inList = ["x", "y", "z", "m"]
		self._outList = ["ld_out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)
		self.m = Holders.ChildAttributeHolder (bridge, self.list)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("m:"), 4, 0)
		self.list.layout.addWidget (self.m, 4, 1)


class DensityList (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Density List"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = ""
		self._inList = ["x", "y", "z", "mass"]
		self._outList = ["dl_out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)

		self._neighbors = PyQt4.QtGui.QSpinBox ()
		self._neighbors.setRange (0, 100)
		self._neighbors.setSingleStep (1)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("Neighbors:"), 4, 0, 1, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self._neighbors, 4, 2, 1, 2, PyQt4.QtCore.Qt.AlignLeft)

	def clear (self):
		AbstractStellarDynamics.clear (self)
		self._neighbors.setValue (0)

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inAttributes = ""
		inResult = self.list.getInAccesses (False)
		if inResult == False:
			return
		for attr in inResult:
			inAttributes += ", " + str (attr)
		outAttributes = ""
		outResult = self.list.getOutNames (False)
		if outResult == False:
			return
		for attr in outResult:
			outAttributes = str (attr)

		if len (inResult) == 4 and len (outResult) == 1:
			commands = []
			commands.append (str (outAttributes) +
				" = density_list (" + str (inAttributes[2:]) + ", " + \
				str (self._neighbors.value ()) + ")")
			self._bridge._ipyShell.BatchCommands (commands, True)
		self.clear ()


class DensityCenter (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._funcName = "density_center"
		self._displayName = "Density Center"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = ""
		self._inList = ["x", "y", "z", "dens_list"]
		self._outList = ["dens_center"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)


class RadialProfile (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Radial Profile"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = ""
		self._inList = ["x", "y", "z", "a"]
		self._outList = ["out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)
		self.mass = Holders.ChildAttributeHolder (bridge, self.list)
		PyQt4.QtCore.QObject.connect (self.mass, PyQt4.QtCore.SIGNAL ("textChanged (const QString&)"), self.checkMass)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("mass (opt):"), 4, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.mass, 4, 1)
		self.massRange = Holders.ChildListHolder (bridge, self.list)
		self.massRange.setEnabled (False)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("mass range:"), 5, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.massRange, 5, 1, PyQt4.QtCore.Qt.AlignLeft)

		self.bins = PyQt4.QtGui.QSpinBox ()
		self.bins.setRange (1, 999999999)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("bins:"), 2, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.bins, 2, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmin = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmin.setRange (0.0, 999999999.0)
		self.rmin.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmin:"), 3, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmin, 3, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmax = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmax.setRange (0.0, 999999999.0)
		self.rmax.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmax:"), 4, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmax, 4, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.scale = PyQt4.QtGui.QCheckBox ("log scale?")
		self.list.layout.addWidget (self.scale, 5, 2, 1, 2)

	def checkMass (self, text):
		if   str (text) == "None":
			self.massRange.setEnabled (False)
		else:
			self.massRange.setEnabled (True)

	def clear (self):
		AbstractStellarDynamics.clear (self)
		self.mass.clear ()
		self.massRange.clear ()
		self.bins.setValue (1)
		self.rmin.setValue (0.0)
		self.rmax.setValue (0.0)

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inResult = self.list.getInAccesses (False)
		outResult = self.list.getOutNames (False)
		if len (inResult) != 4:
			return
		if len (outResult) != 1:
			return

		scale = ""
		if self.scale.checkState () == PyQt4.QtCore.Qt.Checked:
			scale = "log"

		commands = []
		if  self.massRange.isEnabled ():
			if self.mass._item != None and self.massRange._item != None:
				commands.append (str (outResult[0]) +
					" = radial_profile (" +	str (inResult[0]) + ", " + \
					str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
					str (self.mass._item._shellAccess) + ", " + \
					str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
					str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
					", " + self.massRange._item._shellAccess + ", \"" + scale + "\")")
		else:
			commands.append (str (outResult[0]) + \
				" = radial_profile (" + str (inResult[0]) + ", " + \
				str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
				str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
				str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
				", \"" + scale + "\")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()


class DensityProfile (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Density Profile"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = ""
		self._inList = ["x", "y", "z", "mass"]
		self._outList = ["out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)
		self.massRange = Holders.ChildListHolder (bridge, self.list)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("m range:"), 4, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.massRange, 4, 1, PyQt4.QtCore.Qt.AlignLeft)

		comboList = PyQt4.QtCore.QStringList ("mn")
		comboList.append ("mass")
		comboList.append ("number")
		self.compute = PyQt4.QtGui.QComboBox ()
		self.compute.addItems (comboList)
		self.list.layout.addWidget (self.compute, 1, 2, 1, 2, PyQt4.QtCore.Qt.AlignHCenter)

		self.bins = PyQt4.QtGui.QSpinBox ()
		self.bins.setRange (1, 999999999)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("bins:"), 2, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.bins, 2, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmin = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmin.setRange (0.0, 999999999.0)
		self.rmin.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmin:"), 3, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmin, 3, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmax = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmax.setRange (0.0, 999999999.0)
		self.rmax.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmax:"), 4, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmax, 4, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.scale = PyQt4.QtGui.QCheckBox ("log scale?")
		self.list.layout.addWidget (self.scale, 5, 2, 1, 2)

	def clear (self):
		AbstractStellarDynamics.clear (self)
		self.massRange.clear ()
		self.bins.setValue (1)
		self.rmin.setValue (0.0)
		self.rmax.setValue (0.0)

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inResult = self.list.getInAccesses (False)
		outResult = self.list.getOutNames (False)
		if len (inResult) != 4:
			return
		if len (outResult) != 1:
			return

		scale = ""
		if self.scale.checkState () == PyQt4.QtCore.Qt.Checked:
			scale = "log"
		compute = str (self.compute.itemText (self.compute.currentIndex ()))

		commands = []
		if  self.massRange._item == None:
			commands.append (str (outResult[0]) + \
				" = density_profile (" + str (inResult[0]) + ", " + \
				str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
				str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
				str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
				", \"" + compute + "\", \"" + scale + "\")")
		else:
			commands.append (str (outResult[0]) + \
				" = density_profile (" + str (inResult[0]) + ", " + \
				str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
				str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
				str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
				", " + self.massRange._item._shellAccess  + ", \"" + \
				compute + "\", \"" + scale + "\")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()


class DispProfile (AbstractStellarDynamics):
	def __init__ (self, parent, bridge):
		self._displayName = "Disp Profile"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = \
			""
		self._inList = ["x", "y", "z", "A"]
		self._outList = ["out"]
		AbstractStellarDynamics.__init__ (self, parent, bridge)
		self.massRange = Holders.ChildListHolder (bridge, self.list)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("scale:"), 4, 0, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.massRange, 4, 1, PyQt4.QtCore.Qt.AlignLeft)

		comboList = PyQt4.QtCore.QStringList ("log")
		comboList.append ("normal")
		self.compute = PyQt4.QtGui.QComboBox ()
		self.compute.addItems (comboList)
		self.list.layout.addWidget (self.compute, 1, 2, 1, 2, PyQt4.QtCore.Qt.AlignHCenter)

		self.bins = PyQt4.QtGui.QSpinBox ()
		self.bins.setRange (1, 999999999)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("bins:"), 2, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.bins, 2, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmin = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmin.setRange (0.0, 999999999.9)
		self.rmin.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmin:"), 3, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmin, 3, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.rmax = PyQt4.QtGui.QDoubleSpinBox ()
		self.rmax.setRange (0.0, 999999999.9)
		self.rmax.setDecimals (Constants.DOUBLE_PRECISION)
		self.list.layout.addWidget (PyQt4.QtGui.QLabel ("rmax:"), 4, 2, PyQt4.QtCore.Qt.AlignRight)
		self.list.layout.addWidget (self.rmax, 4, 3, PyQt4.QtCore.Qt.AlignLeft)
		self.scale = PyQt4.QtGui.QCheckBox ("log scale?")
		self.list.layout.addWidget (self.scale, 5, 2, 1, 2)

	def clear (self):
		AbstractStellarDynamics.clear (self)
		self.massRange.clear ()
		self.bins.setValue (1)
		self.rmin.setValue (0.0)
		self.rmax.setValue (0.0)

	def perform (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		inResult = self.list.getInAccesses (False)
		outResult = self.list.getOutNames (False)
		if len (inResult) != 4:
			return
		if len (outResult) != 1:
			return

		scale = ""
		if self.scale.checkState () == PyQt4.QtCore.Qt.Checked:
			scale = "log"
		compute = str (self.compute.itemText (self.compute.currentIndex ()))

		commands = []
		if  self.massRange._item == None:
			commands.append (str (outResult[0]) + \
				" = density_profile (" + str (inResult[0]) + ", " + \
				str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
				str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
				str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
				", \"" + compute + "\", \"" + scale + "\")")
		else:
			commands.append (str (outResult[0]) + \
				" = density_profile (" + str (inResult[0]) + ", " + \
				str (inResult[1]) + ", " + str (inResult[2]) + ", " + \
				str (inResult[3]) + ", " + str (self.bins.value ()) + ", " + \
				str (self.rmin.value ()) + ", " + str (self.rmax.value ()) + \
				", " + self.massRange._item._shellAccess  + ", \"" + \
				compute + "\", \"" + scale + "\")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()
