from PyQt4.QtGui  import *
from PyQt4.QtCore import *

import Constants
import Plotting
import Transforms
import Cosmology
import GeneralFunctions
import Statistics
import StellarDynamics
import LinearAlgebra

class Dock (QWidget):
	def __init__ (self, parent, bridge):
		QWidget.__init__ (self)
		self.layout = QHBoxLayout (self)

		# attach it to the bridge
		self._bridge = bridge
		bridge.setDock (self)

		# Invisible Dialogs
		self._gnuplot = Plotting.Gnuplot (self.layout, bridge)
		self._visit = Plotting.VisItPlot (self.layout, bridge)
		QObject.connect (bridge, SIGNAL ("updateVisIt (PyQt_PyObject, PyQt_PyObject, PyQt_PyObject, PyQt_PyObject)"), self._visit.Update)
		self._pylab = Plotting.PyLab (self.layout, bridge)

		self._hubble             = Cosmology.Hubble (self.layout, bridge)
		self._redshifttodistance = Cosmology.RedshiftToDistance (self.layout, bridge)
		self._redshifttogrowth   = Cosmology.RedshiftToGrowth (self.layout, bridge)

		self._equatorialTOxyz  = Transforms.Equatorial2XYZ (self.layout, bridge)
		self._xyzTOequatorial  = Transforms.XYZ2Equatorial (self.layout, bridge)
		self._polarTOxy        = Transforms.Polar2XY (self.layout, bridge)
		self._xyTOpolar        = Transforms.XY2Polar (self.layout, bridge)
		self._sphericalTOxyz   = Transforms.Spherical2XYZ (self.layout, bridge)
		self._xyzTOspherical   = Transforms.XYZ2Spherical (self.layout, bridge)
		self._cylindricalTOxyz = Transforms.Cylindrical2XYZ (self.layout, bridge)
		self._xyzTOcylindrical = Transforms.XYZ2Cylindrical (self.layout, bridge)
		self._fft              = Transforms.FFT (self.layout, bridge)
		self._convolve         = Transforms.Convolve (self.layout, bridge)
		self._computedensity   = Transforms.ComputeDensity (self.layout, bridge)

		self._mean              = Statistics.Mean (self.layout, bridge)
		self._standarddeviation = Statistics.StandardDeviation (self.layout, bridge)
		self._histogram         = Statistics.Histogram (self.layout, bridge)

		self._matrixinvert         = LinearAlgebra.MatrixInvert (self.layout, bridge)
		self._matrixmult           = LinearAlgebra.MatrixMult (self.layout, bridge)
		self._matrixcreate         = LinearAlgebra.MatrixCreate (self.layout, bridge)
		self._matrixcreateidentity = LinearAlgebra.IdentityMatrixCreate (self.layout, bridge)

		self._centerofmass   = StellarDynamics.CenterOfMass (self.layout, bridge)
		self._localdensity   = StellarDynamics.LocalDensity (self.layout, bridge)
		self._densitylist    = StellarDynamics.DensityList (self.layout, bridge)
		self._densitycenter  = StellarDynamics.DensityCenter (self.layout, bridge)
		self._radialprofile  = StellarDynamics.RadialProfile (self.layout, bridge)
		self._densityprofile = StellarDynamics.DensityProfile (self.layout, bridge)
		self._dispprofile    = StellarDynamics.DispProfile (self.layout, bridge)

		self._abs               = GeneralFunctions.AbsoluteValue (self.layout, bridge)
		self._pow               = GeneralFunctions.Pow (self.layout, bridge)
		self._sqrt              = GeneralFunctions.SquareRoot (self.layout, bridge)
		self._sort = GeneralFunctions.Sort (self.layout, bridge)
		self._indexsort = GeneralFunctions.IndexSort (self.layout, bridge)
		self._log = GeneralFunctions.Log (self.layout, bridge)
		self._log10 = GeneralFunctions.Log10 (self.layout, bridge)
		self._exp = GeneralFunctions.Exp (self.layout, bridge)
		self._exp10 = GeneralFunctions.Exp10 (self.layout, bridge)
		self._sin = GeneralFunctions.Sin (self.layout, bridge)
		self._cos = GeneralFunctions.Cos (self.layout, bridge)
		self._cosh = GeneralFunctions.Cosh (self.layout, bridge)
		self._sinh = GeneralFunctions.Sinh (self.layout, bridge)
		self._acos              = GeneralFunctions.Acos (self.layout, bridge)
		self._asin              = GeneralFunctions.Asin (self.layout, bridge)
		self._acosh             = GeneralFunctions.Acosh (self.layout, bridge)
		self._asinh             = GeneralFunctions.Asinh (self.layout, bridge)
		self._atan              = GeneralFunctions.Atan (self.layout, bridge)
		self._attributerandom   = GeneralFunctions.AttributeRandom (self.layout, bridge)
		self._attributeconstant = GeneralFunctions.AttributeConstant (self.layout, bridge)
		self._attributeindex    = GeneralFunctions.AttributeIndex (self.layout, bridge)
		self._real              = GeneralFunctions.Real (self.layout, bridge)
		self._imaginary         = GeneralFunctions.Imaginary (self.layout, bridge)

		self.Init (self.layout)

		# default with nothing docked
		self._current = None
		self._currentAction = None

		# add it to the parent's self.layout
		parent.addWidget (self)

	def DisableAction (self, action):
		self.clear ()
		action.setEnabled (False)
		self._currentAction = action

	# Removes any widgets in the dock, as well as their right-click menu
	def clear (self):
		if self._currentAction != None:
			self._currentAction.setEnabled (True)
			self._currentAction = None
		if self._current != None:
			self._current.setVisible (False)
			self._current = None
		else:
			self._initwidget.setVisible (False)

	def Init (self, parent):
		self._initwidget = QWidget ()
		self.layout = QVBoxLayout ()
		self._initwidget.setLayout (self.layout)


		picLabel = QLabel ()
		picLabel.setPixmap (QPixmap (Constants.BASE_DIR + "/basin.gif", "GIF"))
		picLabel.setAlignment (Qt.AlignHCenter)

		self.layout.addWidget (picLabel)
		parent.addWidget (self._initwidget)

	# Returns the widget that is currently docked
	def GetCurrentWidget (self):
		return self._current

	def ShowDockWidget (self):
		self._current.setVisible (True)
		try:
			self._current.guessDefaults ()
		except AttributeError:
			pass
		try:
			self._current.DefaultCheck ()
		except AttributeError:
			pass

	# Calling this sets the dock widget to the equatorial to cartesian dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def EquatorialToXyz (self):
		if self._current != self._equatorialTOxyz:
			self._current = self._equatorialTOxyz
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cartesian to equatorial dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def XyzToEquatorial (self):
		if self._current != self._xyzTOequatorial:
			self._current = self._xyzTOequatorial
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the polar to cartesian dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def PolarToXy (self):
		if self._current != self._polarTOxy:
			self._current = self._polarTOxy
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cartesian to polar dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def XyToPolar (self):
		if self._current != self._xyTOpolar:
			self._current = self._xyTOpolar
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the spherical to cartesian dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def SphericalToXyz (self):
		if self._current != self._sphericalTOxyz:
			self._current = self._sphericalTOxyz
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cartesian to spherical dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def XyzToSpherical (self):
		if self._current != self._xyzTOspherical:
			self._current = self._xyzTOspherical
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cylindrical to cartesian dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def CylindricalToXyz (self):
		if self._current != self._cylindricalTOxyz:
			self._current = self._cylindricalTOxyz
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cartesian to cylindrical dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def XyzToCylindrical (self):
		if self._current != self._xyzTOcylindrical:
			self._current = self._xyzTOcylindrical
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the gnuplot dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Gnuplot (self):
		if self._current != self._gnuplot:
			self._current = self._gnuplot
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the visit dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def VisIt (self):
		if self._current != self._visit:
			self._current = self._visit
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the gnuplot dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def PyLab (self):
		if self._current != self._pylab:
			self._current = self._pylab
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the visit dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Hubble (self):
		if self._current != self._hubble:
			self._current = self._hubble
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the visit dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def RedshiftToDistance (self):
		if self._current != self._redshifttodistance:
			self._current = self._redshifttodistance
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the visit dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def RedshiftToGrowth (self):
		if self._current != self._redshifttogrowth:
			self._current = self._redshifttogrowth
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the gnuplot dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Fft (self):
		if self._current != self._fft:
			self._current = self._fft
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the convolve dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Convolve (self):
		if self._current != self._convolve:
			self._current = self._convolve
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the convolve dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def ComputeDensity (self):
		if self._current != self._computedensity:
			self._current = self._computedensity
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the sort dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def IndexSort (self):
		if self._current != self._indexsort:
			self._current = self._indexsort
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the sort dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Sort (self):
		if self._current != self._sort:
			self._current = self._sort
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the absolute value dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def AbsoluteValue (self):
		if self._current != self._abs:
			self._current = self._abs
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the log dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Log (self):
		if self._current != self._log:
			self._current = self._log
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the log10 dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Log10 (self):
		if self._current != self._log10:
			self._current = self._log10
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the exp dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Exp (self):
		if self._current != self._exp:
			self._current = self._exp
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the exp10 dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Exp10 (self):
		if self._current != self._exp10:
			self._current = self._exp10
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the sqrt dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def SquareRoot (self):
		if self._current != self._sqrt:
			self._current = self._sqrt
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the sin dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Sin (self):
		if self._current != self._sin:
			self._current = self._sin
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cos dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Cos (self):
		if self._current != self._cos:
			self._current = self._cos
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the cosh dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Cosh (self):
		if self._current != self._cosh:
			self._current = self._cosh
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the sinh dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Sinh (self):
		if self._current != self._sinh:
			self._current = self._sinh
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the acos dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Acos (self):
		if self._current != self._acos:
			self._current = self._acos
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the asin dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Asin (self):
		if self._current != self._asin:
			self._current = self._asin
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the acosh dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Acosh (self):
		if self._current != self._acosh:
			self._current = self._acosh
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the asinh dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Asinh (self):
		if self._current != self._asinh:
			self._current = self._asinh
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the atan dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Atan (self):
		if self._current != self._atan:
			self._current = self._atan
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the attribute_random dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def AttributeConstant (self):
		if self._current != self._attributeconstant:
			self._current = self._attributeconstant
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the attribute_random dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def AttributeRandom (self):
		if self._current != self._attributerandom:
			self._current = self._attributerandom
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the attribute_index dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def AttributeIndex (self):
		if self._current != self._attributeindex:
			self._current = self._attributeindex
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the power dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Pow (self):
		if self._current != self._pow:
			self._current = self._pow
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the real dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Real (self):
		if self._current != self._real:
			self._current = self._real
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the imaginary dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Imaginary (self):
		if self._current != self._imaginary:
			self._current = self._imaginary
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the mean dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Mean (self):
		if self._current != self._mean:
			self._current = self._mean
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the standard deviation dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def StandardDeviation (self):
		if self._current != self._standarddeviation:
			self._current = self._standarddeviation
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the histogram dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def Histogram (self):
		if self._current != self._histogram:
			self._current = self._histogram
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the center of mass dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def CenterOfMass (self):
		if self._current != self._centerofmass:
			self._current = self._centerofmass
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the local density dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def LocalDensity (self):
		if self._current != self._localdensity:
			self._current = self._localdensity
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the density list dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def DensityList (self):
		if self._current != self._densitylist:
			self._current = self._densitylist
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the density center dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def DensityCenter (self):
		if self._current != self._densitycenter:
			self._current = self._densitycenter
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the radial profile dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def RadialProfile (self):
		if self._current != self._radialprofile:
			self._current = self._radialprofile
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the density profile dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def DensityProfile (self):
		if self._current != self._densityprofile:
			self._current = self._densityprofile
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the disp profile dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def DispProfile (self):
		if self._current != self._dispprofile:
			self._current = self._dispprofile
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the matrix invert dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def MatrixInvert (self):
		if self._current != self._matrixinvert:
			self._current = self._matrixinvert
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the matrix multiply dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def MatrixMult (self):
		if self._current != self._matrixmult:
			self._current = self._matrixmult
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the matrix create dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def MatrixCreate (self):
		if self._current != self._matrixcreate:
			self._current = self._matrixcreate
			self.ShowDockWidget ()

	# Calling this sets the dock widget to the matrix create dialog
	# and grabs appropriate menu from the menubar, which gets sent back on a right click
	def MatrixCreateIdentity (self):
		if self._current != self._matrixcreateidentity:
			self._current = self._matrixcreateidentity
			self.ShowDockWidget ()
