from PyQt4.QtGui  import *
from PyQt4.QtCore import *

import re
import sys
import os
from StringIO import StringIO
import string

import Constants

import IPython

# Much of the ipython code here is taken from http://ipython.scipy.org/moin/Cookbook/EmbeddingInGTK
# which is released under the BSD license
# 
# It provides the backend to a pygtk application, which I have used for a PyQt4 frontend

class IPShell:
	def __init__ (self, user_ns = None, user_global_ns = None, cin = None, \
	cout = None, cerr = None, input_func = None):
		if input_func:
			IPython.iplib.raw_input_original = input_func
		if cin:
			IPython.Shell.Term.cin = cin
		if cout:
			IPython.Shell.Term.cout = cout
		if cerr:
			IPython.Shell.Term.cerr = cerr

		# This is to get rid of the blockage that accurs during 
		# IPython.Shell.InteractiveShell.user_setup()
		IPython.iplib.raw_input = lambda x: None

		self.term = IPython.genutils.IOTerm(cin=cin, cout=cout, cerr=cerr)
		os.environ['TERM'] = 'dumb'
		excepthook = sys.excepthook 
		self.IP = IPython.Shell.make_IPython ([], user_ns = user_ns,
			user_global_ns = user_global_ns, embedded = True,
			shell_class = IPython.Shell.InteractiveShell)
		self.IP.system = lambda cmd: self.shell (self.IP.var_expand (cmd), \
			header = 'IPython system call: ', verbose = self.IP.rc.system_verbose)
		sys.excepthook = excepthook
		self.iter_more = 0
		self.history_level = 0
		self.complete_sep =  re.compile('[\s\{\}\[\]\(\)]')

	def execute(self):
		orig_stdout = sys.stdout
		sys.stdout = IPython.Shell.Term.cout
		try:
			line = self.IP.raw_input (None, self.iter_more)
			if self.IP.autoindent:
				self.IP.readline_startup_hook(None)
		except KeyboardInterrupt:
			self.IP.write('\nKeyboardInterrupt\n')
			self.IP.resetbuffer()
			self.IP.outputcache.prompt_count -= 1
        
			if self.IP.autoindent:
				self.IP.indent_current_nsp = 0
			self.iter_more = 0
		except:
			self.IP.showtraceback()
		else:
			self.iter_more = self.IP.push(line)
			if (self.IP.SyntaxTB.last_syntax_error and
				self.IP.rc.autoedit_syntax):
				self.IP.edit_syntax_error()
		if self.iter_more:
			self.prompt = stripColor (str(self.IP.outputcache.prompt2).strip())
			if self.IP.autoindent:
				self.IP.readline_startup_hook(self.IP.pre_readline)
		else:
			self.prompt = stripColor (str(self.IP.outputcache.prompt1).strip())
		sys.stdout = orig_stdout


	def shell (self, cmd,verbose=0,debug=0,header=''):
		stat = 0
		if verbose or debug: print header+cmd
		if not debug:
			input, output = os.popen4(cmd)
			output.close()
			input.close()

	def complete (self, text):
		return self.IP.complete (text)


class CommandHistory (QTextEdit):
	def __init__ (self, shell, bridge):
		"""
		CommandHistory.__init__ (self, shell, bridge)

		This initialization sets up a normal QTextEdit, then
		sets up a bunch of color/font/size settings to be used
		later. After the default settings are loaded, it will
		attempt to load them from the configuration file C{~/.basin/client/ipython_appearance.conf}
		"""
		QTextEdit.__init__ (self)
		self._bridge = bridge
		self._shell = shell
		self.setTextInteractionFlags (Qt.TextBrowserInteraction)
		shell.addWidget (self)
		shell.setStretchFactor (self, 8)

		# Defaults if no settings file is found
		self._bgColor = "#ffffff"
		self._commandSize = "4"
		self._commandColor = "#000000"
		self._commandFont = "Courier"
		self._commandDecorations = ["b"]
		self._resultSize = "4"
		self._resultColor = "#20aa20"
		self._resultFont = "Courier"
		self._resultDecorations = []
		self._commentSize = "3"
		self._commentColor = "#808080"
		self._commentFont = "Courier"
		self._commentDecorations = ["i"]
		self.loadSettings ()

	def scrollToBottom (self):
		self.moveCursor (QTextCursor.End)
		self.ensureCursorVisible ()

	def correctString (self, string):
		"""
		CommandHistory.correctString (self, string)

		This will simply return a string, given a string.
		It is 'corrected' in the sense that it is passed a
		normal string, however, it will be re-formatted
		to be used as html.  This means newlines become <br>,
		spaces become &nbsp;, etc...
		"""
		string = str (string)
		string = string.replace ("<", "&lt;")
		string = string.replace (">", "&gt;")
		string = string.replace ("\n", "<br>")
		string = string.replace (" ", "&nbsp;")
		return string

	def writeCommand (self, string):
		"""
		CommandHistory.writeCommand (self, string)

		Commands are text strings that either the user has
		typed and entered via the CommandLine, or commands
		that were pragmatically entered (and decided that
		user should be informed of).

		This method takes a given string and displays it on
		itself (a QTextEdit) given the pre-loaded settings
		of what a "Command" looks like.  These settings are
		loaded at startup, or anytime the user edits them
		in the preferences menu.  Note that editing these
		display settings will NOT retroactively change
		previous commands written.

		Settings may be found in the following file: C{~/.basin/client/ipython_appearance.conf}
		"""
		string = self.correctString (string)

		dec1, dec2 = "", ""
		for dec in self._commandDecorations:
			dec1 += "<" + dec + ">"
		count = len (self._commandDecorations) - 1
		while count > 0:
			dec2 += "</" + self._commandDecorations[count] + ">"
			count -= 1

		self.append ("<font class=\"Command\" size=\"" + \
			self._commandSize + "\" color=\"" + self._commandColor + "\" face=\"" + \
			self._commandFont + "\">" + dec1 + string + dec2 + "</font>")
		self.scrollToBottom ()

	def writeResult (self, string):
		"""
		CommandHistory.writeResult (self, string)

		Results are what the ipython environment returns
		after having received a command.  Quite often this
		includes the command entered as well (notably 
		ipython's magic commands will not include the
		command as a part of the output).  There is an
		optional setting, kept in the bridge, to reduce
		the output displayed.  If this option is set then
		the results are more of an abridged version of the
		ipython output (notably that output won't be 
		for every node in use, just one).  Otherwise results
		are exact copies, with their color having been
		stripped, of ipython environment results.

		This method takes a given string and displays it on
		itself (a QTextEdit) given the pre-loaded settings
		of what a "Result" looks like.  These settings are
		loaded at startup, or anytime the user edits them
		in the preferences menu.  Note that editing these
		display settings will NOT retroactively change
		previous results written.

		Settings may be found in the following file: C{~/.basin/client/ipython_appearance.conf}
		"""
		string = self.correctString (string)

		# Option set in preferences:
		#   It will prevent the cluster from generating 
		#   output for each node, but rather output one
		#   nodes data and rename it from `0` to `Cluster`
		if self._bridge._limitOut:
			index = string.find ("[1]&nbsp;")
			if index != -1:
				string = string[:index]
			string = string.replace ("[0]", "[Cluster]").replace ("&lt;Results&nbsp;List&gt;<br>", "")

		dec1, dec2 = "", ""
		for dec in self._resultDecorations:
			dec1 += "<" + dec + ">"
		count = len (self._resultDecorations) - 1
		while count > 0:
			dec2 += "</" + self._resultDecorations[count] + ">"
			count -= 1

		self.append ("<font class=\"Result\"size=\"" + \
			self._resultSize + "\" color=\"" + self._resultColor + "\" face=\"" + \
			self._resultFont + "\">" + dec1 + string + dec2 + "</font>")
		self.scrollToBottom ()

	def writeComment (self, string):
		"""
		CommandHistory.writeComment (self, string)

		Comments (probably not the best name to describe
		these) are messages that aren't associated with
		a normal python/ipython session.  They are messages
		this gui has decided to display in the ipython box.
		For instance, if you enter a command with 'def',
		'for', 'if', or 'while', then it will tell you that
		you are in multi-line mode, and that you need to
		enter a blank line to get out of it.  Not a python
		command, but certainly pertinant to ipython.

		This method takes a given string and displays it on
		itself (a QTextEdit) given the pre-loaded settings
		of what a "Comment" looks like.  These settings are
		loaded at startup, or anytime the user edits them
		in the preferences menu.  Note that editing these
		display settings will NOT retroactively change
		previous comments written.

		Settings may be found in the following file: C{~/.basin/client/ipython_appearance.conf}
		"""
		string = self.correctString (string)

		dec1, dec2 = "", ""
		for dec in self._commentDecorations:
			dec1 += "<" + dec + ">"
		count = len (self._commentDecorations) - 1
		while count > 0:
			dec2 += "</" + self._commentDecorations[count] + ">"
			count -= 1

		self.append ("<font class=\"Comment\"size=\"" + self._commentSize + "\" color=\"" + \
			self._commentColor + "\" face=\"" + self._commentFont + "\">" + \
			dec1 + string + dec2 + "</font>")
		self.scrollToBottom ()

	def writeMetaComment (self):
		"""
		CommandHistory.writeMetaComment (self)

		This method prints out a standard message to the command
		history stating that the command entered is a meta-command,
		one which isn't an actual python command, but that the client
		will break up into a bunch of commands to be executed.  This
		is merely a message stating this as a comment.
		"""
		self._shell._commandHistory.writeComment ("Entered a basin meta-command for plotting...")
		self._shell._commandHistory.writeComment ("Note that this isn't a valid python command," + \
			" but is converted into several valid commands.")

	def mousePressEvent (self, ev):
		"""
		CommandHistory.mousePressEvent (self, ev)

		This method is called whenever the user clicks on the CommandHistory.
		It is only looking to see if it was a right click, in which case it
		calls rightClick, otherwise behaves like a normal QTextEdit.
		"""
		if ev.button () == Qt.RightButton:
			self.rightClick (ev.globalPos ())
		else:
			QTextEdit.mousePressEvent (self, ev)

	def keyPressEvent (self, ev):
		"""
		CommandHistory.keyPressEvent (self, ev)

		This method is called whenever the user presses on a key
		while the CommandHistory is in focus.  It is only searching
		to see if the `+` or `-` keys were pressed.

		 - `+` Increases the font size
		 - `-` Decreases the font size
		 Both will do this to everything in the entire CommandHistory.

		If netiher were pressed, it proceeds to call the standard QTextEdit's
		keyPressEvent to behave normally.
		"""

		key = ev.key ()
		if   key == Qt.Key_Plus:
			self.zoomIn ()
		elif key == Qt.Key_Minus:
			self.zoomOut ()
		else:
			QTextEdit.keyPressEvent (self, ev)

	def rightClick (self, position):
		"""
		CommandHistory.rightClick (self, position)

		This method will be called whenever the user performs
		a right click.  It will display three options in a menu
		at the point `position`, where the user had clicked.

		The options are to:
		 - Save the current history to a log file
		 - Increase the font size
		 - Decrease the font size
		"""

		menu = QMenu ("Log")
		menu.addAction (QIcon (Constants.ICON_DIR + "/file/python.png"), \
			"Save IPython Log", self._bridge.SaveIPythonSession)
		menu.addSeparator ()
		menu.addAction (QIcon (Constants.ICON_DIR + "/misc/zoomin.png"), \
			"Increase Font Size", self.zoomIn)
		menu.addAction (QIcon (Constants.ICON_DIR + "/misc/zoomout.png"), \
			"Decrease Font Size", self.zoomOut)
		menu.addSeparator ()
		menu.addAction ("Copy", self.copy)
		menu.exec_ (position)

	def zoomIn (self):
		"""
		CommandHistory.zoomIn (self)

		This method is attached to the '+' (note it needs shift) key as
		well as a right-click menu when right-clicking on the CommandHistory.

		It will increase everything in the history's font size by 1.
		"""

		QTextEdit.zoomIn (self)

	def zoomOut (self):
		"""
		CommandHistory.zoomIn (self)

		This method is attached to the '-' key as well as a right-click
		menu when right-clicking on the CommandHistory.

		It will decrease everything in the history's font size by 1.
		"""
		QTextEdit.zoomOut (self)

	def loadSettings (self):
		file = QFile (str (QDir.home ().path ()) + "/.basin/client/ipython_appearance.conf")
		if file.exists ():
			file.open (QIODevice.ReadOnly | QIODevice.Text)
			stream = QTextStream (file)

			reBGColor = re.compile ("\s*BGCOLOR\s*:\s*.*\s*")

			reSize = re.compile ("\s*.*\s*:\s*SIZE\s*:\s*.*\s*")
			reFont = re.compile ("\s*.*\s*:\s*FONT\s*:\s*.*\s*")
			reColor = re.compile ("\s*.*\s*:\s*COLOR\s*:\s*.*\s*")
			reDecorations = re.compile ("\s*.*\s*:\s*DECORATIONS\s*:\s*.*\s*")

			reCommand = re.compile ("\s*COMMAND\s*:\s*.*\s*:\s*.*\s*")
			reResult = re.compile ("\s*RESULT\s*:\s*.*\s*:\s*.*\s*")
			reComment = re.compile ("\s*COMMENT\s*:\s*.*\s*:\s*.*\s*")

			while not stream.atEnd ():
				line = str (stream.readLine ())
				val = line[line.rfind (":") + 1:]
				if   reCommand.match (line):
					if   reSize.match (line):
						self._commandSize = val
					elif reFont.match (line):
						self._commandFont = val
					elif reColor.match (line):
						self._commandColor = val
					elif reDecorations.match (line):
						self._commandDecorations = val.split (",")
				elif reResult.match (line):
					if   reSize.match (line):
						self._resultSize = val
					elif reFont.match (line):
						self._resultFont = val
					elif reColor.match (line):
						self._resultColor = val
					elif reDecorations.match (line):
						self._resultDecorations = val.split (",")
				elif reComment.match (line):
					if   reSize.match (line):
						self._commentSize = val
					elif reFont.match (line):
						self._commentFont = val
					elif reColor.match (line):
						self._commentColor = val
					elif reDecorations.match (line):
						self._commentDecorations = val.split (",")
				elif reBGColor.match (line):
					self._bgColor = val
			file.close ()


class CommandLine (QLineEdit):
	def __init__ (self, shell, bridge):
		self._bridge = bridge
		self._autopx = False
		self._shell = shell
		self._history = []
		self._historyCount = -1
		self._update = False
		self._singleLine = True
		self._visitStart = False
		QLineEdit.__init__ (self)
		self.setFocusPolicy (Qt.ClickFocus)
		self.setReadOnly (True)
		shell.addWidget (self)
		shell.setStretchFactor (self, 2)
		self._multiLineHistoryBuffer = []

		QObject.connect (shell._commandStatus.toggleButton, SIGNAL ("clicked ()"), self.setFocus)

	def keyPressEvent (self, ev):
		key = ev.key ()
		if key == Qt.Key_Enter or key == Qt.Key_Return:
			self.enterCommand ()
		elif key == Qt.Key_Up:
			self.upHistory ()
		elif key == Qt.Key_Down:
			self.downHistory ()
		elif key == Qt.Key_AsciiTilde:
			print self._bridge._ipyShell.complete (self.text ())
		else:
			QLineEdit.keyPressEvent (self, ev)


	# Should be called everytime a Command (non-silent) is issued,
	# it accumulates these for re-use
	def addToHistory (self):
		self._historyCount = -1
		command = self.text ()
		self._history.insert (0, command)

	# traverse backwards-in-time for issued commands
	# currently no limit on how many are stored
	def upHistory (self):
		self._historyCount += 1
		if self._historyCount >= len (self._history):
			self._historyCount = len (self._history) - 1
		if self._historyCount != -1:
			self.setText (self._history[self._historyCount])

	# traverse forwards-in-time for issued commands
	# it catches any attempt to go "down" from the current line
	def downHistory (self):
		self._historyCount -= 1
		if self._historyCount < 0:
			self._historyCount = -1
			self.clear ()
		else:
			self.setText (self._history[self._historyCount])

	def enterCommand (self, autoUpdate = True):
		command = str (self.text ())
		self.addToHistory ()

		# There are commands entered before a session is 'initialized'.  They don't need to
		# be recorded, because for the buffer history to be useful, you must already be
		# initialized
		if self._bridge._ipyShell._initialized:
			# Store the original command, there's a good possibility things will
			# be done before this gets executed...
			command = str (self.text ())
			if self._singleLine:
				# If there was a multi-line statement before this one, then there are
				# a bunch of commands that were stuck into this buffer (because they
				# couldn't be appended to the server's history).  It is now safe to
				# append them, so do that first
				while len (self._multiLineHistoryBuffer) > 0:
					buffer = self._multiLineHistoryBuffer.pop (0)
					self._bridge._ipyShell.ControllerCommand ("buffer_history.append (\"" +
						self._bridge._user + "@" + buffer.replace ("\n",
						"\\\\\n").replace ("\"", "\\\\\"").replace ("\'", "\\\\\'") +
						"\")", False)
				# And then append the actual command you just entered to the sever's history
				self._bridge._ipyShell.ControllerCommand ("buffer_history.append (\"" +
					self._bridge._user + "@" + command.replace ("\n",
					"\\\\\n").replace ("\"", "\\\\\"").replace ("\'", "\\\\\'") +
					"\")", False)
			else:
				# If its here, its a multi-line statement, so its history gets tacked to
				# a buffer until you're back to single-line mode
				self._multiLineHistoryBuffer.append (command)
			# Restore the original command to be executed
			self.setText (command)

		self.filter ()
		rv = self._shell._processLine ()

		# Write command & result to CommandHistory,
		# Then clear out the commandline
		self._shell._commandHistory.writeCommand (str (self.text ()))
		self._shell._commandHistory.writeResult (rv)

		if   rv == "" and self._bridge._parallel:
			self._shell._commandHistory.writeComment ("Multi-line: enter a blank line to finish")
			self._singleLine = False
		elif rv != "":
			self._singleLine = True

		if self._singleLine:
			self.determineUpdate (autoUpdate)

		self.clear ()
		return rv

	def enterSilentCommand (self, autoUpdate = True):
		self.filter ()
		rv = self._shell._processLine ()
		self.clear ()

		if self._singleLine:
			self.determineUpdate (autoUpdate)
		return rv

	def determineUpdate (self, autoUpdate):
		if   not autoUpdate:
			self._update = False
		elif self._update and self._bridge._connected \
		and self._bridge._parallel:
			self._update = False
			self.setUpdateLine ()
			self._bridge.Update ()

	def setUpdateLine (self):
		self.setText ("print \"\\n\" + BASIN_UPDATE")
		result = self.enterSilentCommand (False).split ("\n")
		lineNumber =  result[1][result[1].find ("[", 1) + \
			1:result[1].find ("]", result[1].find ("[", 1))]
		self.setText ("BASIN_UPDATE = '" + lineNumber + "'")
		self.enterSilentCommand (False)

	def filter (self):
		if (not self._singleLine) or self.text () == "":
			return True

		self.setText (str (self.text ()).strip ())

		if self.filterExit ():
			return True
		if self.filterVisitStart ():
			return True
		# This only applies for "one-touch" visit, which requires the visit python module
		if self._bridge._dock._visit._visitModuleLoaded:
			if self.filterVisitExpose ():
				return True
		if self.filterBuffer ():
			return True
		if self.filterAutopx ():
			return True
		if self._shell._initialized == True and self._bridge._parallel and self._bridge._connected:
			if self.filterPlotting ():
				return False
			if self.filterDataModification ():
				return True
			if self.filterRegions ():
				return True
			if self.filterVars ():
				return True
			if self.filterGenericAssignment ():
				return True


	# I'm not really happy with the way this is handled right now,
	# but for now it manages the number of times visit_module_start ()
	# is executed.  For it to work properly, it should only ever be
	# done once per session.
	def filterVisitStart (self):
		command = str (self.text ())
		if command.find ("visit_module_start") != -1 and command.find ("@") == -1:
			if self._bridge._visitModuleStarted:
				self.clear ()
			else:
				result = self._shell.getVisitModuleStatus ()
				if   result == "False":
					self._bridge._ipyShell.SilentCommand ( \
						"BASIN_VISIT_START = True")
					self.setText ("visit_module_start ()")
					return True
				else:
					self._bridge._visitModuleStarted = True
					self.clear ()
					return False
		return False


	# This will populate host-side variables consisting of the plot names that have been exposed
	def filterVisitExpose (self):
		discreteRegExp = re.compile ("visit_expose_discrete\s\(.*\)")
		continuousRegExp = re.compile ("visit_expose_continuous\s\(.*\)")
		command = str (self.text ())
		if discreteRegExp.match (command):
			quote2 = command.rfind ("\"")
			quote1 = command[:quote2].rfind ("\"") + 1
			name = command[quote1:quote2]
			exposeList, windowList = self._bridge._ipyShell.getVisItExposeDiscrete ()

			_append = True
			for each in exposeList:
				if each == name:
					_append = False
			if _append:
				exposeList.append (name)
				windowList.append (str (self._bridge._dock._visit._visit._windowCount))
			discreteString = ""
			for index in xrange (len (exposeList)):
				discreteString += exposeList[index] + ":" + windowList[index] + "\\n"
			self.setText ("BASIN_VISIT_DISCRETE_LIST = \"" + discreteString + "\"")
			self.enterSilentCommand (False)

			self.setText (command)

			if self._bridge._importVisIt:
				# Starts thread if it hasn't been (includes launching & connecting)
				if not self._bridge._dock._visit.StartVisItThread ():
					# If the thread was already started (gives an initial window), add a window manually
					self._bridge._dock._visit.emit (SIGNAL ("threadAddWindow ()"))
					self._bridge._dock._visit._visitWindows += 1
					windows = self._bridge._dock._visit._visitWindows
					self._bridge._dock._visit.emit (SIGNAL ("threadSetActiveWindow (PyQt_PyObject)"), windows)
					self._bridge._dock._visit.emit (SIGNAL ("threadReOpenSimulation ()"))
				self._bridge._dock._visit.emit (SIGNAL ("threadAddPlotPseudocolor (PyQt_PyObject)"), name)
				self._bridge._dock._visit.emit (SIGNAL ("threadDrawPlots ()"))
			self._update = True
			return True
		if continuousRegExp.match (command):
			quote2 = command.rfind ("\"")
			quote1 = command[:quote2].rfind ("\"") + 1
			name = command[quote1:quote2]
			exposeList, windowList = self._bridge._ipyShell.getVisItExposeContinuous ()

			_append = True
			for each in exposeList:
				if each == name:
					_append = False
			if _append:
				exposeList.append (name)
				windowList.append (str (self._bridge._dock._visit._visit._windowCount))
			continuousString = ""
			for index in xrange (len (exposeList)):
				continuousString += exposeList[index] + ":" + windowList[index] + "\\n"
			self.setText ("BASIN_VISIT_CONTINUOUS_LIST = \"" + continuousString + "\"")
			self.enterSilentCommand (False)

			self.setText (command)

			if self._bridge._importVisIt:
				# Starts thread if it hasn't been (includes launching & connecting)
				if not self._bridge._dock._visit.StartVisItThread ():
					# If the thread was already started (gives an initial window), add a window manually
					self._bridge._dock._visit.emit (SIGNAL ("threadAddWindow ()"))
					self._bridge._dock._visit._visitWindows += 1
					windows = self._bridge._dock._visit._visitWindows
					self._bridge._dock._visit.emit (SIGNAL ("threadSetActiveWindow (PyQt_PyObject)"), windows)
					self._bridge._dock._visit.emit (SIGNAL ("threadReOpenSimulation ()"))
				self._bridge._dock._visit.emit (SIGNAL ("threadAddPlotVolume (PyQt_PyObject)"), name)
				self._bridge._dock._visit.emit (SIGNAL ("threadDrawPlots ()"))
			self._update = True
			return True
		return False

	def filterPlotting (self):
		command = str (self.text ()).replace (" ", "").replace ("\t", "")
		expPylab = "plot\(.*,\"[Pp][Yy][Ll][Aa][Bb].*\)"
		if re.compile (expPylab).match (str (command)):
			self._bridge._ipyShell._commandHistory.writeCommand (str (self.text ()))
			command = command [command.index ("(") + 1:command.rindex (")")]
			lastParen = command.rindex ("\"")
			secondlastParen = command.rindex ("\"", 0, lastParen - 1)

			options = command [secondlastParen + 1:lastParen].split (",")
			data = command[:secondlastParen - 1].split (",")
			style = ""

			if data[-1].find ("PLOT_STYLE=") != -1:
				style = data.pop ()
				equals = style.index ("=")
				style = ", \"" + style[equals + 1:] + "\""

			# Make sure there are atleast 2 options, otherwise it can't be performed
			if len (options) < 2:
				self.setText ("")
				return True

			self._shell._commandHistory.writeMetaComment ()

			if options[1] == "scatter":
				self._bridge.CursorBusy ()
				# Get data types for each data input, and if they aren't basin Attributes
				# or python lists, then get out of here, because this function is useless
				types = []
				for each in data:
					type = self._bridge._ipyShell.getVariableType (each)
					if type != "list" and type != "_basin.Attribute":
						self.setText ("")
						return
					types.append (type) 

				# Convert all attributes to lists, leave the lists alone...
				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + str (index) + " = " + \
							data[index] + ".to_list ()")

				# This section must be done locally (note the only way to get to this point
				# was to be in parallel)
				self._bridge._ipyShell.toggleAutopx ()

				# Pull the data from the cluster to the remote side, the naming conventions
				# are a tad different depnding on whether a data conversion had to be made
				# As variables are being pulled to the local side, tack them onto a string
				# which will serve as an arguments list to a function.
				_dataString = ""
				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + str (index) + \
							" = rc.pull (0, '" + "BASIN_TEMP_" + str (index) + "')[0]")
						_dataString += "BASIN_TEMP_" + str (index) + ","
					else:
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + data[index] + \
							" = rc.pull (0, '" + data[index] + "')[0]")
						_dataString += "BASIN_TEMP_" + data[index] + ","
				_dataString = _dataString[:-1]

				# the second element of options better be the plot type, i haven't implemented
				# a way to check this as of yet.
				self._bridge._ipyShell.Command ("pylab.plot (" + _dataString + style + \
					")", False)

				if len (options) > 2:
					for command in PyLabOptionsParse (options[2:]):
						self._bridge._ipyShell.Command (command, False)

				self._bridge._ipyShell.Command ("pylab.xlabel (\"" + data[0].replace ("\"", "\\\"") + \
					"\")", False)
				self._bridge._ipyShell.Command ("pylab.ylabel (\"" + data[1].replace ("\"", "\\\"") + \
					"\")", False)

				self._bridge.CursorNorm ()
				self._bridge._ipyShell.Command ("pylab.show ()", False)
				self._bridge.CursorBusy ()

				# Wipe off local temporary variables!
				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + str (index) + ")")
					else:
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + data[index] + ")")

				# Go back to the parallel side and delete any temporary variables you
				# may have made
				self._bridge._ipyShell.toggleAutopx ()
				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + str (index) + ")")
				self._bridge.CursorNorm ()


			elif options[1] == "contour" or options[1] == "pcolor":
				self._bridge.CursorBusy ()
				# Get data types for each data input, and if they aren't basin Attributes
				# or python lists, then get out of here, because this function is useless
				types = []
				for each in data:
					type = self._bridge._ipyShell.getVariableType (each)
					print type
					if type != "pylab.array" and type != "_basin.Attribute":
						self.setText ("")
						return
					types.append (type) 

				# Convert all attributes to lists, leave the lists alone...
				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + str (index) + " = " + \
							data[index] + ".to_list ()")
						self._bridge._ipyShell.Command ("BASIN_NX_TEMP_" +str(index) + "="+\
							data[index]+".get_dims()")

				self._bridge._ipyShell.toggleAutopx ()

				#if len (data) == 3:
				#	self._bridge._ipyShell.Command ("BASIN_TEMP_0 = [[], [], []]")
				#	for index in xrange (len (data)):
				#		if types[index] == "_basin.Attribute":
				#			self._bridge._ipyShell.Command ("BASIN_TEMP_0[" + \
				#				str (index) + "] = rc.pull (0, '" + "BASIN_TEMP_" + \
				#				str (index) + "')[0]")
				#		else:
				#			self._bridge._ipyShell.Command ("BASIN_TEMP_0[" + \
				#				data[index] + "] = rc.pull (0, '" + \
				##				data[index] + "')[0]")
				#	self._bridge._ipyShell.Command ("pylab.contour (BASIN_TEMP_0)", False)
				if len (data) == 1:
					if types[0] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_0" + \
							" = rc.pull (0, 'BASIN_TEMP_0')[0]")
						self._bridge._ipyShell.Command ("BASIN_NX_TEMP_0" + \
							" = rc.pull (0, 'BASIN_NX_TEMP_0')[0]")
						# Now, convert to an array
						self._bridge._ipyShell.Command ("BASIN_ARR_TEMP=pylab.array(BASIN_TEMP_0)")
						self._bridge._ipyShell.Command ("BASIN_ARR_TEMP.shape=BASIN_NX_TEMP_0")
					#else:
					#	self._bridge._ipyShell.Command ("BASIN_TEMP_" + \
					#		data[0] + " = rc.pull (0, '" + \
					#		data[0] + "')[0]")
					self._bridge._ipyShell.Command ("pylab."+options[1]+" (BASIN_ARR_TEMP)", False)


				if len (options) > 2:
					for command in PyLabOptionsParse (options[2:]):
						self._bridge._ipyShell.Command (command, False)

				if len (data) == 3:
					self._bridge._ipyShell.Command ("pylab.xlabel (\"" + \
						data[0].replace ("\"", "\\\"") + "\")", False)
					self._bridge._ipyShell.Command ("pylab.ylabel (\"" + \
						data[1].replace ("\"", "\\\"") + "\")", False)

				self._bridge.CursorNorm ()
				self._bridge._ipyShell.Command ("pylab.show ()", False)
				self._bridge.CursorBusy ()
				self._bridge._ipyShell.Command ("del (BASIN_TEMP_0)", False)

				self._bridge._ipyShell.toggleAutopx ()

				for index in xrange (len (data)):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + str (index) + ")")
				self._bridge.CursorNorm ()


			elif options[1] == "hist":
				self._bridge.CursorBusy ()
				# Get data types for each data input, and if they aren't basin Attributes
				# or python lists, then get out of here, because this function is useless
				type = self._bridge._ipyShell.getVariableType (data[0])
				if type != "list" and type != "_basin.Attribute":
					self.setText ("")
					return

				if type == "_basin.Attribute":
					self._bridge._ipyShell.Command ("BASIN_TEMP_0 = " + \
						data[0] + ".to_list ()")

				self._bridge._ipyShell.toggleAutopx ()

				if type == "_basin.Attribute":
					self._bridge._ipyShell.Command ("BASIN_TEMP_0 = rc.pull (0, '" + \
						"BASIN_TEMP_0')[0]")
				else:
					self._bridge._ipyShell.Command ("BASIN_TEMP_0 = rc.pull (0, '" + \
						data[0] + "')[0]")

				self._bridge._ipyShell.Command ("pylab.hist (BASIN_TEMP_0, " + data[1] + ")")
				self._bridge.CursorNorm ()
				self._bridge._ipyShell.Command ("pylab.show ()")
				self._bridge.CursorBusy ()

				self._bridge._ipyShell.Command ("del (BASIN_TEMP_0)")
				self._bridge._ipyShell.toggleAutopx ()
				if type == "_basin.Attribute":
					self._bridge._ipyShell.Command ("del (BASIN_TEMP_0)")
				self._bridge.CursorNorm ()
			elif options[1] == "errorbar":
				self._bridge.CursorBusy ()
				# Get data types for each data input, and if they aren't basin Attributes
				# or python lists, then get out of here, because this function is useless
				types = []
				for each in data[:-2]:
					type = self._bridge._ipyShell.getVariableType (each)
					if type != "list" and type != "_basin.Attribute":
						self.setText ("")
						return
					types.append (type) 

				# Convert all attributes to lists, leave the lists alone...
				for index in xrange (len (data[:-2])):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + str (index) + " = " + \
							data[index] + ".to_list ()")

				# This section must be done locally (note the only way to get to this point
				# was to be in parallel)
				self._bridge._ipyShell.toggleAutopx ()

				# Pull the data from the cluster to the remote side, the naming conventions
				# are a tad different depnding on whether a data conversion had to be made
				# As variables are being pulled to the local side, tack them onto a string
				# which will serve as an arguments list to a function.
				_dataString = ""
				for index in xrange (len (data[:-2])):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + str (index) + \
							" = rc.pull (0, '" + "BASIN_TEMP_" + str (index) + "')[0]")
						_dataString += "BASIN_TEMP_" + str (index) + ","
					else:
						self._bridge._ipyShell.Command ("BASIN_TEMP_" + data[index] + \
							" = rc.pull (0, '" + data[index] + "')[0]")
						_dataString += "BASIN_TEMP_" + data[index] + ","
				_dataString = _dataString[:-1]
				_dataString += ", " + data[-2] + ", " + data[-1]

				# the second element of options better be the plot type, i haven't implemented
				# a way to check this as of yet.
				self._bridge._ipyShell.Command ("pylab.errorbar (" + _dataString + style + \
					")", False)

				if len (options) > 2:
					for command in PyLabOptionsParse (options[2:]):
						self._bridge._ipyShell.Command (command, False)

				self._bridge._ipyShell.Command ("pylab.xlabel (\"" + data[0].replace ("\"", "\\\"") + \
					"\")", False)
				self._bridge._ipyShell.Command ("pylab.ylabel (\"" + data[1].replace ("\"", "\\\"") + \
					"\")", False)

				self._bridge.CursorNorm ()
				self._bridge._ipyShell.Command ("pylab.show ()", False)
				self._bridge.CursorBusy ()

				# Wipe off local temporary variables!
				for index in xrange (len (data[:-2])):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + str (index) + ")")
					else:
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + data[index] + ")")

				# Go back to the parallel side and delete any temporary variables you
				# may have made
				self._bridge._ipyShell.toggleAutopx ()
				for index in xrange (len (data[:-2])):
					if types[index] == "_basin.Attribute":
						self._bridge._ipyShell.Command ("del (BASIN_TEMP_" + str (index) + ")")
				self._bridge.CursorNorm ()
			else:
				self._shell._commandHistory.writeComment ("PyLab plot type '" + options[1] + "' unkown!")

			# Wow... you converted one pseudo command into a zillion different commands...
			# Remember that the actual command entered has yet to be executed, so whatever
			# gets set to the command line now is still going to be executed! (For now I
			# blank it out)
			self.setText ("")
			return True
		return False

	def filterGenericAssignment (self):
		"""
		CommandLine.filterGenericAssignment (self)

		This method is a standard filter method which is meant
		to catch any assignments being made.  All the filter does
		is ensure that the bridge will update itself, as it should
		happen after data has potentially been altered.
		"""
		assignRegExp = re.compile (".*\s*=\s*.*")
		command = str (self.text ())
		if assignRegExp.match (command):
			self._update = True
			return True
		return False

	def filterExit (self):
		"""
		CommandLine.filterExit (self)

		This method is a standard filter method which is meant
		to catch the user from using the exit function/command
		to exit from the ipython shell.  Doing this will actually
		make the client hang for eternity, so users must close
		the actual client to stop the ipython session.
		"""
		exitRegExp = re.compile ("\s*exit\s*(\(\s*\))?\s*")
		command = str (self.text ())
		if exitRegExp.match (command):
			return True
		return False

	def filterBuffer (self):
		command = str (self.text ())
		if command.find ("buffer_history") != -1:
			return True
		return False

	def filterAutopx (self):
		command = str (self.text ()).strip ()
		if command.find ("%autopx") != -1 or command.find ("autopx") != -1:
			if self._bridge._parallel == True:
				self._bridge.parallelOff ()
			else:
				self._bridge.parallelOn ()
			return True
		return False

	def filterDataModification (self):
		command = str (self.text ())

		# Regular expression for a method being called with one argument that is a string
		exp = ".*(delete_grid|delete_list|delete_attribute|set_name)\s*\(\s*\".*\"\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for a method being called with one argument that is not a string
		exp = ".*(delete_child|add_child|fft|del|power_spectrum)\s*\(\s*.*\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for a method being called with two arguments that are strings
		exp = ".*(change_attribute_key)\s*\(\s*\".*\"\s*\,\s*\".*\"\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for a method being called with two arguments that are not strings
		exp = ".*(convolve)\s*\(\s*.*\s*\,\s*.*\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for a method being called with two arguments, one a string, the other not
		exp = ".*(add_list|add_grid|add_attribute)\s*\(\s*\".*\"\s*\,\s*.*\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for transformations
		exp = ".*(equatorial_to_xyz|xyz_to_equatorial" + \
		      "|spherical_to_xyz|xyz_to_spherical" + \
		      "|cylindrical_to_xyz|xyz_to_cylindrical)" + \
		      "\s*\(\s*.*\s*,\s*\".*\"\s*\,\s*\".*\"\s*,\s*\".*\"\s*,\s*\".*\"\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for transformations
		exp = ".*(equatorial_to_xyz|xyz_to_equatorial" + \
		      "|spherical_to_xyz|xyz_to_spherical" + \
		      "|cylindrical_to_xyz|xyz_to_cylindrical)" + \
		      "\s*\(\s*.*\s*\,\s*.*\s*,\s*.*\s*,\s*.*\s*\)\s*"

		if re.compile (exp).match (str (command)):
			self._update = True
			return
		# Regular expression for transformations
		exp = ".*(polar_to_xy|xy_to_polar)" + \
		      "\s*\(\s*.*\s*,\s*\".*\"\s*\,\s*\".*\"\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for transformations
		exp = ".*(polar_to_xy|xy_to_polar)" + \
		      "\s*\(\s*.*\s*,\s*.*\s*\,\s*.*\s*\)\s*"
		if re.compile (exp).match (str (command)):
			self._update = True
			return

		# Regular expression for cosmology, 1 non-string. 2 strings, 2 non-strings
		exp = ".*(redshift_to_distance)" + \
		      "\s*\(\s*.*\s*,\s*\".*\"\s*,\s*\".*\"\s*,\s*.*\s*\,\s*.*\s*\)\s*"

		if re.compile (exp).match (str (command)):
			self._update = True
			return

	def filterRegions (self):
		# Preserve original ipython command, one will be entered first
		commandOrig = str (self.text ())
		command = commandOrig.replace (" ", "").replace ("\t", "")
		regionList = self._bridge._ipyShell.getTopLevelRegions ()
		equalIndex = command.find ("=")
		if equalIndex != -1:
			# should be a regular expression of the form:
			# 	something = Region ("something")
			# but with more liberal whitespacing
			exp = ".*=\s*Region\s*\(\s*.*\s*\)\s*"
			if re.compile (exp).match (str (command)):
				var = command[:equalIndex]
				_append = True
				for each in regionList:
					if each == var:
						_append = False
				if _append:
					regionList.append (var)
				toplevelString = ""
				for each in regionList:
					toplevelString += each + "\\n"
				self.setText ("BASIN_TOPLEVEL_REGION_LIST = \"" + toplevelString + "\"")
				self.enterSilentCommand (False)

				self.setText ( var + " = " + var)
				self.filterVars ()

				self.setText (commandOrig)
				self._update = True
				return True
		self.setText (commandOrig)
		return False

	def filterVars (self):
		# Preserve original ipython command, two will be entered first
		commandOrig = str (self.text ())
		command = commandOrig.replace (" ", "").replace ("\t", "")
		equalIndex = command.find ("=")
		if command.find ("BASIN_") == -1 and equalIndex != -1 and command[equalIndex + 1] != "=" \
		and equalIndex < (len (command) - 1) and command[:equalIndex].find ("[") == -1:
			self.setText ("BASIN_FREE_VARIABLE_LIST += \"" + command[:equalIndex] + "\\n\"")
			self.enterSilentCommand (False)
			reference = command[equalIndex + 1:]
			real_ref = ""
			for i in xrange (len (reference)):
				if not re.compile ("\s").match (reference[i]):
					if re.compile ("\"").match (reference[i]):
						real_ref += "\\\""
					else:
						real_ref += reference[i]
			self.setText ("BASIN_FREE_VARIABLE_LIST += \"-" + real_ref + "-\\n\"")
			self.enterSilentCommand (False)
			self.setText (commandOrig)
			self._update = True
			return True
		return False

	def dropEvent (self, ev):
		if self._bridge._drugItem and ev.mimeData ().hasFormat ("text/plain"):
			self.insert (self._bridge._drugItem._shellAccess)

			self._bridge.PopDrugItem ()
			self.setFocus ()

			if ev.source () == self._bridge._regionViewer:
				ev.setDropAction (Qt.LinkAction)
				ev.accept ()
			else:
				ev.acceptProposedAction ()
		else:
			ev.ignore ()

     	def dragEnterEvent (self, ev):
		ev.accept ()

	def dragMoveEvent (self, ev):
		ev.accept ()


class CommandStatus (QLabel):
	def __init__ (self, shell):
		QLabel.__init__ (self)
		self._shell = shell

		layout = QHBoxLayout ()
		shell.addLayout (layout)
		layout.addWidget (self)

		self.toggleButton  = Constants.BQPushButton ("Toggle Parallel Mode", layout)
		self.toggleButton.setEnabled (False)
		QObject.connect (self.toggleButton, SIGNAL ("clicked ()"), shell.toggleAutopx)

	def showStatus (self, string):
		self.setText (QString (str (string)))

	def toggleSingleLine (self):
		self._shell._commandLine._singleLine = not self._shell._commandLine._singleLine
		if self._shell._commandLine._singleLine:
			self._shell.Command ("", False)


class ConsoleView (QVBoxLayout):
	def __init__ (self, layout, bridge):
		QVBoxLayout.__init__ (self)
		self._commandHistory = CommandHistory (self, bridge)
		self._commandStatus = CommandStatus (self)
		self._commandLine = CommandLine (self, bridge)

	def setMenuBar (self, menubar):
		self._commandHistory.setMenuBar (menubar)

	def write(self, text, editable=False):
		self.showReturned (unicode (text))

	def showPrompt(self, prompt):
		self._commandHistory.append ("")

	def showStatus (self, string):
		self._commandStatus.showStatus (string)

	def showReturned(self, text):
		for line in stripColor (text).split ("\n"):
			self._commandHistory.writeResult (line)

	def getHistory (self):
		return self._commandLine._history

	def raw_input (self, prompt = ''):
		return str (self._commandLine.text ().toUtf8 ())

	def _processLine (self):
		self.history_pos = 0
		self.execute()
		rv = self.cout.getvalue()
		while rv.find ("PBConnectionLost:") != -1:
			self.execute()
			rv = self.cout.getvalue()
		self.cout.truncate(0)
		return stripColor (rv.strip ("\n"))


class IPythonView(ConsoleView, IPShell):
	def __init__(self, layout, bridge):
		ConsoleView.__init__(self, layout, bridge)
		self._scriptPath = None

		self._bridge = bridge
		bridge.setIPyShell (self)

		shell = QGroupBox ("IPython Shell")
		shell.setLayout (self)
		layout.addWidget (shell)
		self._initialized = False


	def Init (self, host, port, user):
		self.cout = StringIO()
		IPShell.__init__(self, cout=self.cout,cerr=self.cout, 
                             input_func=self.raw_input)
		self.execute()
		self.cout.truncate(0)
		self._host = host
		self._port = port
		self._user = user

		self.Command ("import subprocess")

		self.attemptImportPylab ()
		self.attemptImportNumpy ()
		self.attemptImportScipy ()

		self.Command ("import ipython1.kernel.api as kernel")
		self.Command ("rc = kernel.RemoteController(('" + str (host) + "' ," + str (port) + "))")
		self.Command ("rc.activate()")
		self._bridge._nodes = self.SilentCommand ("print \"\\n\" + str (rc.getIDs ())")\
			[1:-1].replace (" ", "").split (",")
		self._bridge._numNodes = len (self._bridge._nodes)
		self.toggleAutopx ()

		# This is the first time a command is issued across the cluster,
		# if it fails from a socket.error, then the connection failed
		connectCheck = self.Command ("import sys, os")
		if connectCheck.find ("error") != -1:
			# Get back to local commands
			self.toggleAutopx ()

			# Generate error dialog
			error = QDialog (self._bridge._mainWindow)
			error.setWindowTitle ("Connection Error")
			errorLayout = QVBoxLayout ()
			error.setLayout (errorLayout)
			errorLayout.addWidget (QLabel (QString (
				"Connection Failure!\n\n" +
				"Please ensure that you entered\n" +
				"a valid server name and port.\n\n" +
				"You should also check to make\n" +
				"sure that the basin_start.sh\n" +
				"script is running properly on\n" +
				"the cluster server.\n\n" +
				"You may try connecting again.")))
			acceptButton = Constants.BQPushButton ("Ok", errorLayout)
			QObject.connect (acceptButton, SIGNAL ("clicked ()"), error.accept)
			error.exec_ ()
			return False

		self._bridge.Connect ()
		self.Command ("sys.setdlopenflags(258)")
		self.Command ("from basin import *")
		self._scriptPath = self.ScriptPath ()
		self.ControllerCommand ("sys.path.append(\"" + self._scriptPath + "\")")
		self.ControllerCommand ("import buffer_history, filesystem")

		_update = False

		result = self.SilentCommand ("print BASIN_TOPLEVEL_REGION_LIST")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_TOPLEVEL_REGION_LIST = ''")
		else:
			_update = True

		result = self.SilentCommand ("print BASIN_FREE_VARIABLE_LIST")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_FREE_VARIABLE_LIST = ''")
		else:
			_update = True

		result = self.SilentCommand ("print BASIN_VISIT_DISCRETE_LIST")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_VISIT_DISCRETE_LIST = ''")
		else:
			_update = True

		result = self.SilentCommand ("print BASIN_VISIT_CONTINUOUS_LIST")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_VISIT_CONTINUOUS_LIST = ''")
		else:
			_update = True

		result = self.SilentCommand ("print BASIN_UPDATE")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_UPDATE = '0'")
		else:
			_update = True

		result = self.SilentCommand ("print COSMOLOGY")
		if result.find ("NameError") != -1:
			self.SilentCommand ("COSMOLOGY = cosmology ()")

		result = self.SilentCommand ("print CONVOLVE")
		if result.find ("NameError") != -1:
			self.SilentCommand ("CONVOLVE = ConvKernel ()")

		result = self.SilentCommand ("print BASIN_VISIT_START")
		if result.find ("NameError") != -1:
			self.SilentCommand ("BASIN_VISIT_START = False")

		if _update:
			self._bridge.Update ()
		else:
			self.ControllerCommand ("buffer_history.delete ()")


		self._initialized = True
		return True

	def attemptImportNumpy (self):
		"""
		IPythonView.attemptImportNumpy (self)

		This function will try to import the module
		`numpy` onto the local machine.  It will record
		its  success in the bridge's ._importNumpy
		so that the client knows whether or not it can
		handle commands using the `numpy` module or not.

		This method is assumed to be done while local.
		I omit checking for parallelism since this is
		performed in an initialization stage, where
		its status is in limbo.
		"""
		result = self.SilentCommand ("import numpy")
		if result.find ("ImportError") != -1:
			self._bridge._importNumpy = False
		else:
			self._bridge._importNumpy = True

	def attemptImportScipy (self):
		"""
		IPythonView.attemptImportScipy (self)

		This function will try to import the module
		`scipy` onto the local machine.  It will record
		its  success in the bridge's ._importScipy
		so that the client knows whether or not it can
		handle commands using the `scipy` module or not.

		This method is assumed to be done while local.
		I omit checking for parallelism since this is
		performed in an initialization stage, where
		its status is in limbo.
		"""
		result = self.SilentCommand ("import scipy")
		if result.find ("ImportError") != -1:
			self._bridge._importScipy = False
		else:
			self._bridge._importScipy = True

	def attemptImportPylab (self):
		"""
		IPythonView.attemptImportPylab (self)

		This function will try to import the module
		`pylab` onto the local machine.  It will record
		its  success in the bridge's ._importPylab
		so that the client knows whether or not it can
		handle commands using the `pylab` module or not.

		This method is assumed to be done while local.
		I omit checking for parallelism since this is
		performed in an initialization stage, where
		its status is in limbo.
		"""
		result = self.SilentCommand ("import pylab")
		if result.find ("ImportError") != -1:
			self._bridge._importPylab = False
		else:
			self._bridge._importPylab = True

	def Command (self, string, autoUpdate = True):
		# I used to catch if you were in multi-line mode here...
		# It was removed, but keep in mind it was here incase something else
		# goes haywire.
		self._commandLine.setText (str (string))
		return self._commandLine.enterCommand (autoUpdate)

	def SilentCommand (self, string, autoUpdate = True):
		if not self._bridge._ipyShell._commandLine._singleLine:
			self._bridge._statusBar.showTimed ("Can't perform while in multi-line statement...", STATUS_WAIT)
			return
		self._commandLine.setText (str (string))
		return self._commandLine.enterSilentCommand (autoUpdate)

	def ControllerCommand (self, string, autoUpdate = True):
		if not self._bridge._ipyShell._commandLine._singleLine:
			self._bridge._statusBar.showTimed ("Can't perform while in multi-line statement...", STATUS_WAIT)
			return
		alreadyLocal = self._bridge._parallel
		self._bridge.makeLocal ()
		command = "rc.execute (0, \"" + string.replace ("\"", "\\\"") + "\")"
		result = self.SilentCommand (command, autoUpdate)
		if alreadyLocal:
			self._bridge.makeParallel ()
		return result

	# Meant to do a series of Command () calls, generally speaking the last command ought to be caught by
	# the filters and trigger an update, otherwise none will occur
	def BatchCommands (self, commands, finalUpdate = True):
		self._bridge.CursorBusy ()
		for command in commands:
			self.Command (command, False)
		self._bridge.CursorNorm ()
		if finalUpdate:
			self.setUpdateLine ()
			self._bridge.Update ()

	def toggleAutopx (self):
		result = self.Command ("%autopx")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", "toggleAutopx", result)

	def toggleSilentAutopx (self):
		result = self.SilentCommand ("%autopx")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"toggleSilentAutopx", result)

	def VisItPath (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		self.SilentCommand ("BASIN_TEMP_FILE = open (os.getenv ('HOME') + '/.basin/host/visit.path', 'r+')")
		result = self.SilentCommand ("print \"\\n\" + BASIN_TEMP_FILE.read ()[:-1]")
		self.SilentCommand ("BASIN_TEMP_FILE.close ()")
		self.SilentCommand ("del (BASIN_TEMP_FILE)")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", "VisItPath", result)
			return
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i		
		result = str (result[index + 1])
		result = result[:result.rfind ('/')]
		result = result[:result.rfind ('/')]
		return result

	def ScriptPath (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		self.SilentCommand ("BASIN_TEMP_FILE = open (os.getenv ('HOME') + '/.basin/host/client_script.path', 'r+')")
		result = self.SilentCommand ("print \"\\n\" + BASIN_TEMP_FILE.read ()[:-1]")
		self.SilentCommand ("BASIN_TEMP_FILE.close ()")
		self.SilentCommand ("del (BASIN_TEMP_FILE)")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"ScriptPath", result)
			return
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i		
		return str (result[index + 1])

	def getVariableType (self, var):
		result = self.SilentCommand ("print \"\\n\" + str (type (" + var + "))").split ("\n")[3]
		result = result[result.index ("'") + 1:result.rindex ("'")]
		return result

	def getVisitModuleStatus (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		result = self.ControllerCommand ("print \"\\\\n\" + str (BASIN_VISIT_START)", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"getVisitModuleStatus", result)
			return False
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i		
		return str (result[index + 1])

	def getVisItExposeContinuous (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		result = self.ControllerCommand ("print \"\\\\n\" + BASIN_VISIT_CONTINUOUS_LIST", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"getVisItExposeContinuous", result)
			return []
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i
		result = result[index + 1:]

		exposeList = []
		windowList = []
		for line in result:
			colon = line.rfind (":")
			exposeList.append (line[:colon])
			windowList.append (line[colon + 1:])
		return exposeList, windowList

	def getVisItExposeDiscrete (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		result = self.ControllerCommand ("print \"\\\\n\" + BASIN_VISIT_DISCRETE_LIST", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"getVisItExposeDiscrete", result)
			return []
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i
		result = result[index + 1:]

		exposeList = []
		windowList = []
		for line in result:
			colon = line.rfind (":")
			exposeList.append (line[:colon])
			windowList.append (line[colon + 1:])
		return exposeList, windowList


	def getTopLevelRegions (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		result = self.ControllerCommand ("print \"\\\\n\" + BASIN_TOPLEVEL_REGION_LIST", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"getTopLevelRegions", result)
			return []
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i
		return result[index + 1:]

	def removeTopLevelRegion (self, oldname):
		regionList = self.getTopLevelRegions ()
		finalString = ""
		# swap out all occurances of oldname in regionList with newname
		for each in regionList:
			if each != oldname:
				finalString += str (each) + "\\n"
		result = self.SilentCommand ("BASIN_TOPLEVEL_REGION_LIST = \'" + \
			str (finalString) + "\'", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"removeTopLevelRegion", result)

	def removeFreeVariable (self, oldname):
		freeVars, freeAccesses = self.getFreeVariables ()
		finalString = ""
		index = 0
		while index < len (freeVars):
			if freeVars[index] != oldname:
				finalString += str (freeVars[index]) + "\\n"
				finalString += "-" + str (freeAccesses[index]) + "-\\n"
			index += 1
		result = self.SilentCommand ("BASIN_FREE_VARIABLE_LIST = \'" + \
			str (finalString) + "\'", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"removeFreeVariable", result)

	def replaceFreeVariable (self, varName, shellAccess):
		freeVars, freeAccesses = self.getFreeVariables ()
		finalString = ""
		index = 0
		while index < len (freeVars):
			if freeVars[index] != varName:
				finalString += str (freeVars[index]) + "\\n"
				finalString += "-" + str (freeAccesses[index]) + "-\\n"
			else:
				finalString += str (varName) + "\\n"
				finalString += "-" + str (shellAccess) + "-\\n"
			index += 1
		result = self.SilentCommand ("BASIN_FREE_VARIABLE_LIST = \'" + \
			str (finalString) + "\'", False)
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_ipython_shell.py", "IPythonView", \
				"replaceFreeVariable", result)
		
	def getFreeVariables (self):
		# Retrieve the entire free variable storage variable
		# and split it into a list of string lines
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		result = self.ControllerCommand ("print \"\\\\n\" + BASIN_FREE_VARIABLE_LIST", False)
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i		

		# garbage lines
		result = result[index + 1:]

		temp = None
		real_result, real_access = [], []
		for elem in result:
			# This conditional checks for the second line needed for a fv
			# which is the shell access, also denoted by being surrounded by -'s
			if elem[0] == "-" and elem[-1] == "-":
				if len (real_result) == len (real_access) + 1:
					real_access.append (elem[1:-1])
				elif temp:
					real_access[real_result.index (temp)] = elem[1:-1]
					temp = None
			# And if it isn't surrounded by -'s, it is the actual free variable
			else:
				if real_result.count (elem) < 1:
					real_result.append (elem)
				else:
					temp = elem
		return real_result, real_access

	# Checks to see if the command-line has anything in it
	def isClear (self):
		if self._commandLine.text () ==	"":
			return True
		return False

	# So that objects using the bridge don't need to type so much,
	# I extract the method to this higher-level object
	def setUpdateLine (self):
		self._commandLine.setUpdateLine ()


def stripColor (string):
	"""
	stripColor (string):
	
	Takes in a string, and returns that same string with
	all console colors having been removed.
	"""
	result, show, index, L = "", True, 0, len (string)
	while index < L:
		if (not show) and (string[index] in ['m']):
			show = True
		elif show:
			n = string.find('\x1B', index)
			if n == -1:
				return result + string[index:]
			else:
				result += string[index:n]
				index = n
				show = False
		index += 1
	return result


def PyLabOptionsParse (options):
	commands = []
	for option in options:
		if "=" in option:
			equalsIndex = option.index ("=")
			type = option[:equalsIndex]
			value = option[equalsIndex + 1:]
			commands.append ("pylab." + type + " (\"" + value + "\")")
		else:
			commands.append ("pylab." + option + " ()")
			
	return commands
